//SparkContext
import org.apache.spark.SparkContext
import org.apache.spark.SparkConf

//Json4s
import org.json4s._
import org.json4s.jackson.JsonMethods._

//GraphX
import org.apache.spark._
import org.apache.spark.graphx._
import org.apache.spark.rdd.RDD

//Lists
import scala.collection.mutable.ListBuffer
import scala.xml.XML



/***Summary***
TwitterGraph is the specific parser for twitter, it takes a Twitter
conversation or a specific tweets jsons (obtained directly from twitter API)
and SetUp a SocialGraph with this information.
*************/

/***Conversation Json***
There are two main ways to parse a conversation:

1st: Do it tweet by tweet: place on every line a json from a tweet from the
conversation and read it line by line and parse them using parseTweet.

2nd: Using a conversation json that must be composed on that way:
{"tweets":[{jsonTweet1}, {jsonTweet2}, {jsonTweet3}]}
Where a jsonTweet is a json of a tweet obtained directly from twitter API
***********************/

class TwitterGraphFromXML(context: SparkContext ) extends SocialGraph(context){
  var userReferences : Map[String, List[Long]] = Map()
  var focusTo : Map[Long, Option[Long]] = Map().withDefaultValue(None)
  var tweetAuthor : Map[Long, String] = Map()




  def getMeaningOfLabel(relation: String ): String = {
    val meaning = relation match{
      case "ATTACKS" => "attack"
      case "SUPPORTS" => "support"
      case _ => "noone"
    }
    return meaning
  }


  def parseXMLLabelledConversation( xmlLabelledConversation: String ) {
     val xmlconv = XML.loadFile( xmlLabelledConversation )
     // Parse the argument-list list of <arg > elements
     val arguments = xmlconv \ "argument-list"
     val listofarguments = arguments \ "arg"
     for (arg <- listofarguments) {
       //  println( (arg \ "@id").text + arg.text + (arg \ "@weight").text)
       addMessage( (arg \ "@id").text.toLong, "anonymous",
       arg.text, arg.attributes.asAttrMap, (arg \ "@weight").text.toFloat )
     }
     // Parse the argument pairs
     val argpairs =  xmlconv \ "argument-pairs"
     val listofargpairs = argpairs \ "pair"
     for (apair <- listofargpairs) {
       // println( apair )
       val answer = ((apair \ "t") \"@id" ).text
       val source = ((apair \ "h") \"@id" ).text
       // println( answer + " -> " + source + " " + getMeaningOfLabel((apair \ "@entailment").text) )
       addConnection(  answer.toLong, source.toLong, getMeaningOfLabel((apair \ "@entailment").text))
     }
  }


}
