//SparkContext
import org.apache.spark.SparkContext
import org.apache.spark.SparkConf

//GraphX
import org.apache.spark._
import org.apache.spark.graphx._
import org.apache.spark.rdd.RDD

//Lists
import scala.collection.mutable.ListBuffer

class RelationshipsCalcs(){

    /*****Summary*****
    |Graph represents Weight / Defeaters / Finished
    |Defeaters: -1 at initial state, to set it's value at least one time,
    |avoiding mark them as finished
    |Finished: True / False
    *****************/
    def calcDefeaters(sGraph: SocialGraph) : Int = {
      // Init graph to (Weight, -1, False)
      val graph = sGraph.getSubGraph(true, false, false).getRepresentation().FloatIntegerBool(0, -1, false, true, true)

      val sssp = graph.pregel((0, 0, false))(
        (id, main, received) => {
          //Evaluate the joined messages and set the new state
          if((main._2 == received._2 && received._3) || main._3){
            //If the state it's the same as previous mark as finished
            (main._1, received._2, true)
          }
          else{
            (main._1, received._2, false)
          }
        },
        triplet => {
          //Compute the messages to send
          if(triplet.srcAttr._1 >= triplet.dstAttr._1 && //Weight
             triplet.dstAttr._3 == false){               //Dst is not finished
            //Send a message
            if(triplet.srcAttr._2 == 0){
              Iterator((triplet.dstId, (0, 1, triplet.srcAttr._3)))
            }
            else{
              Iterator((triplet.dstId, (0, 0, triplet.srcAttr._3)))
            }
          }
          else{
            //Don't send a message
            Iterator.empty
          }
        },
        (a,b) => {
          //Join messages into one
          (0, a._2 + b._2, a._3 && b._3)
        }
      )
      val defeaterNodes = sssp.vertices.map(node => {
        if(node._2._2 > 0){
          0
        }
        else{
          1
        }
      }).reduce((a, b) => a+b)
      println("TOTAL DEFEATERS: " + defeaterNodes)
      defeaterNodes

    }


    def optimizedCalcDefeaters(sGraph: SocialGraph) : Int = {
      // Init graph to (Weight, -1, False)
      val graph = sGraph.getSubGraph(true, false, false).getRepresentation().FloatIntegerBool(0, -1, false, true, true)

      val sssp = graph.pregel((0, 0, false))(
        (id, main, received) => {
          //Evaluate the joined messages and set the new state
          if((main._2 == received._2) || main._3){
            //If the state it's the same as previous mark as finished
            (main._1, received._2, true)
          }
          else{
            if(!received._3){
              (main._1, received._2, false)
            }
            else{
              (main._1, received._2, true)
            }
          }
        },
        triplet => {
          //Compute the messages to send
          if(triplet.srcAttr._1 >= triplet.dstAttr._1 && //Weight
             triplet.srcAttr._3 && !triplet.dstAttr._3){               //Dst is not finished
            //Send a message
            if(triplet.srcAttr._2 == 0){
              Iterator((triplet.dstId, (0, 1, triplet.srcAttr._3)))
            }
            else{
              Iterator((triplet.dstId, (0, 0, triplet.srcAttr._3)))
            }
          }
          else{
            //Don't send a message
            Iterator.empty
          }
        },
        (a,b) => {
          //Join messages into one
          (0, a._2 + b._2, true)
        }
      )
      val defeaterNodes = sssp.vertices.map(node => {
        if(node._2._2 > 0){
          0
        }
        else{
          1
        }
      }).reduce((a, b) => a+b)
      println("TOTAL DEFEATERS: " + defeaterNodes)
      defeaterNodes
    }



    def calcDefeatersMessageCount(sGraph: SocialGraph) : Long = {
      // Init graph to (Weight, -1, False)
      println("Preparing environment for calculate message count")
      val graph = sGraph.getSubGraph(true, false, false).getRepresentation().FloatIntegerLongBool(0, 0, 0, false, true, true)
      //println("USED VERTICES: " + graph.numVertices)
      val sssp = graph.pregel((0, -1, 0, false))(
        (id, main, received) => {
          //Initial message, avoid mark leafs as unfinished
          if(received._2 == -1){
            (main._1, 0, 0, main._4)
          }
          else{
            //If the state it's the same as previous mark as finished
            (main._1, received._2, main._3 + received._3, main._2 == received._2 || received._4)
          }
        },
        triplet => {
          //Vote to finish
          if(triplet.srcAttr._4 && triplet.dstAttr._4){
            Iterator.empty
          }
          else{
            //Compute the messages to send
            //Send a message
            if(triplet.srcAttr._2 == 0 && triplet.srcAttr._1 >= triplet.dstAttr._1){
              Iterator((triplet.dstId, (0, 1, 1, triplet.srcAttr._4)))
            }
            else{
              Iterator((triplet.dstId, (0, 0, 1, triplet.srcAttr._4)))
            }
          }
        },
        (a,b) => {
          //Join messages into one
          (0, a._2 + b._2, a._3 + b._3, a._4 && b._4)
        }
      )
      val totalMessages = sssp.vertices.reduce((a,b) => (0, (0, 0, a._2._3 + b._2._3, true)))
      println("TOTAL MESSAGES: " + totalMessages._2._3)
      val defeaterNodes = sssp.vertices.map(node => {
        if(node._2._2 > 0){
          0
        }
        else{
          1
        }
      }).reduce((a, b) => a+b)
      println("TOTAL DEFEATERS: " + defeaterNodes)
      defeaterNodes
    }


    def optimizedCalcDefeatersMessageCount(sGraph: SocialGraph ) : scala.xml.Elem = {
      // Graph node state: (Weight, NumAcceptedAttackers, NumReceivedMessages, Finished )
      //
      // Init graph to ((Weight,INdegree,DEFsINdegree), (0,0), 0, False) this is the format for main (vertex state)
      val graph = sGraph.getSubGraph(true, false, false).getRepresentation().FloatIntegerIntegerIntegerLongBool( true, 0, 0, 0, false, true)
      var solutionlist = sGraph.getListOfDisconnectedNodes()

      val sssp = graph.pregel(initialMsg = (0, (0,0), 0),activeDirection = EdgeDirection.Out)(
        (id, main, received) => {
          // Map message "received" by node "id" with current state "main"
          // return new state with same format as main
          //
          //Evaluate the joined messages and set the new state
          // println( "Node "+id.toString()+" State: "+main.toString()+" Message received: "+received.toString())

          val newaccepteddefeaters = main._2._1 + received._2._1
          val newrejecteddefeaters = main._2._2 + received._2._2
          // Check if number of rejected defeaters equals to DEFsINdegree
          if( (newrejecteddefeaters) ==  main._1._3 ){

            // println(" PRE:"+main._2.toString()+"=CURRENT "+received._2.toString()+", vertex "+id.toString()+ " defeaters: "+received._2.toString()+ " is finished ")
            (main._1,  (newaccepteddefeaters ,newrejecteddefeaters), main._3 + received._3, true)
          }
          else{
            // Check if it has at least one accepted defeater
            if ( (newaccepteddefeaters) >= 1){
              // println(" PRE:"+main._2.toString()+"=CURRENT "+received._2.toString()+", vertex "+id.toString()+ " defeaters: "+received._2.toString()+ " is finished ")
              (main._1, (newaccepteddefeaters , newrejecteddefeaters), main._3 + received._3, true )
            }
            else{
              // println( "No final state for : " +id.toString()+ " state:   ("+newaccepteddefeaters.toString()+ " " + newrejecteddefeaters.toString() )
              (main._1, (newaccepteddefeaters , newrejecteddefeaters), main._3 + received._3, false)
            }
          }
        },
        triplet => {
          //Compute the message to send for edge edge triplet
          // triplet.srcAttr, triplet.attr, triplet.dstAttr
          if(triplet.srcAttr._1._1 >= triplet.dstAttr._1._1 && //Weight Source >= Weight Destination
             triplet.srcAttr._4 && !triplet.dstAttr._4){       // Src Finished but Dst is not finished
            //Send a "defeater accepted message" to dstId, if #rejected attackers == INdegree
            if(triplet.srcAttr._2._2 == triplet.srcAttr._1._3){
              // If Source node has ALL defeaters rejected, add one more accepted defeater for Dst
              // println( " MESSAGE: "+ triplet.srcId.toString()+ " -> "+triplet.dstId.toString()+" = (1,0)")
              Iterator((triplet.dstId, (0, (1,0), 1)))
            }
            else { if (triplet.srcAttr._2._1 >= 1) {
              // If Source node has some accepted defeaters, add one more  rejected defeater for Dst
              // println( " MESSAGE: "+ triplet.srcId.toString()+ " -> "+triplet.dstId.toString()+" = (0,1)")
              Iterator((triplet.dstId, (0, (0,1), 1)))
            }
            else{
              //Don't send a message
              Iterator.empty
            } }
          }
          else{
            //Don't send a message
            Iterator.empty
          }
        },
        (a,b) => {
          //Join messages into one
          (0, (a._2._1 + b._2._1, a._2._2 + b._2._2), a._3 + b._3 )
        }
      )
      val totalMessages = sssp.vertices.reduce((a,b) => (0, ((0,0,0), (0,0), a._2._3 + b._2._3, true)))
      println("TOTAL MESSAGES: " + totalMessages._2._3)
      val defeaterNodes = sssp.vertices.map(node => {
        if(node._2._2._2 == node._2._1._3) {  1  }  else{  0  }       }
      ).reduce((a, b) => a+b)

      // node =  (id,((Weight,INdegree), ( accepted,rejected ), messagesrecv, Finished))
      val listofdefeaterNodes = sssp.vertices.filter( node => (node._2._2._2 == node._2._1._3)).
         map( node => {node._1.toString() } ).collect()
      println("total ACCEPTED without disconnected: " + defeaterNodes)

      listofdefeaterNodes.foreach( solutionlist += )
      println("LIST OF ACCEPTED ONES:" )
      solutionlist.foreach( println )
      println("END OF LIST" )
      println("TOTAL ACCEPTED: "+solutionlist.length.toString())
      <ArgumentationAnswers><answer>{solutionlist.map(i => <arg id={i} />)}</answer></ArgumentationAnswers>
    }
  }
